SystemOrganization addCategory: #'Arduino-Core'!
SystemOrganization addCategory: #'Arduino-Core-Devices'!
SystemOrganization addCategory: #'Arduino-Core-Firmata'!
SystemOrganization addCategory: #'Arduino-Core-PinMode'!

Error subclass: #FirmataNotInstalledError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Firmata'!

!FirmataNotInstalledError commentStamp: 'Richo 1/5/2010 15:22' prior: 0!
I'm just an Error class indicating that Firmata is not currently installed on the Arduino board.!

Object subclass: #Arduino
	instanceVariableNames: 'protocol digitalPins analogPins'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!Arduino commentStamp: '<historical>' prior: 0!
I represent the Arduino board. I have a collection of pins and a protocol (i.e. Firmata). 
Below you will find a code example using the Firmata protocol:

---------------------------------------------------------------------------------------
| arduino digitalPin analogPin led button ldr servo |

"Connection"
arduino := Arduino on: Firmata new.
arduino connectOnPort: 'COM5'.
arduino isConnected.
arduino disconnect. "Don't forget to disconnect me after finish using me"

"Digital pins - output"
digitalPin := arduino digitalPin: 4.
digitalPin setOutput.
digitalPin value: 1.

"Digital pins - input"
arduino activateDigitalReports.
digitalPin setInput.
digitalPin value.

"Analog pins (analog pins are input only)"
analogPin := arduino analogPin: 0.
analogPin activate.
analogPin value.

"To avoid configuring pins, you can also attach/detach devices:"

"Led (digital output)"
led := LightEmittingDiode new.
led attach: (arduino digitalPin: 13).
led value: 1.
led value: 0.

"Button (digital input)"
button := EButton new.
button attach: (arduino digitalPin: 2).
button value.

"Ldr (analog input)"
ldr := Photoresistor new.
ldr attach: (arduino analogPin: 5).
ldr value.

"Servo (currently servos can only be attached to pins 9-10)"
servo := Servo new.
servo attach: (arduino digitalPin: 9).
servo angle: 0.
servo angle: 90.
servo angle: 180.

"To see the current list of available devices evaluate:"
AttachableDevice allSubclasses!

!Arduino class methodsFor: 'just for testing' stamp: 'Richo 1/31/2011 09:24'!
instancesConnected
"Just for testing. Do not use!!

Update 31/1/2011: I read this comment after a while and I thought the 'Do not use!!' warning was funny and a little exagerated :). This method was certainly used for testing while developing one of the earlier versions but it does no harm if you execute it."
^self allInstances select: [:each| each isConnected]! !

!Arduino class methodsFor: 'instance creation' stamp: 'Richo 5/31/2010 15:50'!
new
"Firmata is the protocol by default"
^self on: Firmata new! !

!Arduino class methodsFor: 'instance creation' stamp: 'Richo 8/18/2009 16:31'!
on: aProtocol
^(self basicNew setProtocol: aProtocol) initialize! !

!Arduino methodsFor: 'connection' stamp: 'Richo 9/11/2009 17:21'!
activateDigitalReports
protocol activateDigitalPort: 0; activateDigitalPort: 1.
! !

!Arduino methodsFor: 'connection' stamp: 'Richo 9/11/2009 17:24'!
activateReports
self activateDigitalReports.
analogPins do: [:each | each activate]! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 8/31/2009 16:07'!
analogPin: aSmallInteger 
	^ analogPins at: aSmallInteger ifAbsent: [self inaccessiblePinError ]! !

!Arduino methodsFor: 'connection' stamp: 'Richo 1/31/2011 09:59'!
connectOnPort: portName
	self connectOnPort: portName boardType: (ArduinoType at: #ArduinoATmega328)! !

!Arduino methodsFor: 'connection' stamp: 'Richo 1/31/2011 09:59'!
connectOnPort: portName boardType: anArduinoType
	"I reconfigure each pin every time I connect to a board"
	protocol connectOnPort: portName boardType: anArduinoType.
	self reconfigurePins! !

!Arduino methodsFor: 'connection' stamp: 'Richo 9/11/2009 17:21'!
deactivateDigitalReports
protocol deactivateDigitalPort: 0;deactivateDigitalPort: 1.
! !

!Arduino methodsFor: 'connection' stamp: 'Richo 1/12/2010 12:59'!
deactivateReports
self deactivateDigitalReports.
analogPins do: [:each | each deactivate]! !

!Arduino methodsFor: 'accessing' stamp: 'Richo 8/31/2009 16:06'!
digitalPin: aSmallInteger 
	^ digitalPins at: aSmallInteger ifAbsent: [self inaccessiblePinError]! !

!Arduino methodsFor: 'connection' stamp: 'Richo 8/31/2009 15:15'!
disconnect
	protocol disconnect.! !

!Arduino methodsFor: 'errors' stamp: 'Richo 4/6/2011 10:37'!
inaccessiblePinError
self error: 'Inaccessible pin' translated! !

!Arduino methodsFor: 'initialize-release' stamp: 'Richo 8/31/2009 16:44'!
initialize
	digitalPins := Dictionary newFrom: ((2 to: 13)
				collect: [:i | i-> (ArduinoDigitalPin on: protocol number: i)]).
	analogPins :=Dictionary newFrom: ( (0 to: 5)
				collect: [:i | i -> (ArduinoAnalogPin on: protocol number: i)])! !

!Arduino methodsFor: 'connection' stamp: 'Richo 8/24/2009 16:14'!
isConnected
^protocol isConnected! !

!Arduino methodsFor: 'connection' stamp: 'Richo 9/8/2009 10:33'!
reconfigurePins
	digitalPins values , analogPins values
		do: [:each | each refresh]! !

!Arduino methodsFor: 'private' stamp: 'Richo 8/18/2009 16:31'!
setProtocol: aProtocol
protocol := aProtocol ! !

Object subclass: #ArduinoPin
	instanceVariableNames: 'number protocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!ArduinoPin commentStamp: 'Richo 9/15/2009 11:49' prior: 0!
Abstract class. I represent an ArduinoPin!

ArduinoPin subclass: #ArduinoAnalogPin
	instanceVariableNames: 'isActive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!ArduinoAnalogPin commentStamp: 'Richo 9/15/2009 11:52' prior: 0!
I represent an Arduino analog pin. Analog pins are always input. Before I can access my value, I must be activated by sending #activate.!

!ArduinoAnalogPin methodsFor: 'as yet unclassified' stamp: 'Richo 8/31/2009 15:22'!
activate
isActive := true.
self refresh! !

!ArduinoAnalogPin methodsFor: 'as yet unclassified' stamp: 'Richo 8/31/2009 15:23'!
deactivate
	isActive := false.
self refresh.! !

!ArduinoAnalogPin methodsFor: 'initializing' stamp: 'Richo 8/31/2009 15:23'!
initialize
isActive := false.! !

!ArduinoAnalogPin methodsFor: 'as yet unclassified' stamp: 'Richo 8/31/2009 15:24'!
isActive
^isActive! !

!ArduinoAnalogPin methodsFor: 'initializing' stamp: 'Richo 8/31/2009 16:46'!
refresh
	isActive
		ifTrue: [ protocol activateAnalogPin: number]
		ifFalse: [ protocol deactivateAnalogPin: number] ! !

!ArduinoAnalogPin methodsFor: 'as yet unclassified' stamp: 'Richo 8/31/2009 16:46'!
value
^ protocol analogRead: number! !

ArduinoPin subclass: #ArduinoDigitalPin
	instanceVariableNames: 'mode value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!ArduinoDigitalPin commentStamp: 'Richo 9/15/2009 11:55' prior: 0!
I represent an Arduino digital pin. Digital pins can be input or output. To change my mode you must send #mode: with a PinMode as a parameter or you can use #setInput/#setOutput.!

!ArduinoDigitalPin methodsFor: 'initialize-release' stamp: 'Richo 9/7/2009 16:20'!
initialize
mode := OutputMode new.
value := 0.! !

!ArduinoDigitalPin methodsFor: 'mode' stamp: 'Richo 8/19/2009 16:02'!
mode
^mode! !

!ArduinoDigitalPin methodsFor: 'mode' stamp: 'Richo 6/27/2011 15:16'!
mode: aPinMode
	mode := aPinMode.
	self refreshMode! !

!ArduinoDigitalPin methodsFor: 'initialize-release' stamp: 'Richo 8/31/2009 15:21'!
refresh
self refreshMode; refreshValue! !

!ArduinoDigitalPin methodsFor: 'mode' stamp: 'Richo 4/30/2010 16:07'!
refreshMode
	mode refreshModeOn: number using: protocol.! !

!ArduinoDigitalPin methodsFor: 'value' stamp: 'Richo 4/30/2010 18:17'!
refreshValue
	mode isOutput ifTrue: [mode refreshValue: value on: number using: protocol]
! !

!ArduinoDigitalPin methodsFor: 'mode' stamp: 'Richo 9/7/2009 16:36'!
setInput

	self mode: InputMode new! !

!ArduinoDigitalPin methodsFor: 'mode' stamp: 'Richo 9/7/2009 16:20'!
setOutput
self mode: OutputMode new! !

!ArduinoDigitalPin methodsFor: 'value' stamp: 'Richo 3/9/2010 10:50'!
value 

	^ mode isOutput
		ifTrue: [value]
		ifFalse: [value := protocol digitalRead: number]! !

!ArduinoDigitalPin methodsFor: 'value' stamp: 'Richo 8/31/2009 15:19'!
value: aNumber 
	value := aNumber.
self refreshValue! !

!ArduinoPin class methodsFor: 'instance creation' stamp: 'Richo 8/18/2009 17:29'!
new
self shouldNotImplement! !

!ArduinoPin class methodsFor: 'instance creation' stamp: 'Richo 8/31/2009 16:44'!
on: aProtocol number: pinNumber 
	^ (self basicNew setProtocol: aProtocol number: pinNumber) initialize! !

!ArduinoPin methodsFor: 'accessing' stamp: 'Richo 8/24/2009 16:51'!
number
^number! !

!ArduinoPin methodsFor: 'printing' stamp: 'Richo 8/19/2009 14:18'!
printOn: aStream
super printOn: aStream.
aStream nextPutAll: '(', number asString, ')'! !

!ArduinoPin methodsFor: 'initializing' stamp: 'Richo 8/31/2009 15:21'!
refresh
self subclassResponsibility! !

!ArduinoPin methodsFor: 'private' stamp: 'Richo 8/31/2009 16:44'!
setProtocol: aProtocol number: aSmallInteger 
	protocol := aProtocol.
	number := aSmallInteger! !

Object subclass: #ArduinoProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

!ArduinoProtocol commentStamp: 'Richo 3/9/2010 10:34' prior: 0!
Abstract class. I define the protocol to communicate with an Arduino board.!

!ArduinoProtocol methodsFor: 'analog pins' stamp: 'Richo 8/26/2009 16:51'!
activateAnalogPin: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 9/11/2009 13:46'!
activateDigitalPort: portNumber
self subclassResponsibility
! !

!ArduinoProtocol methodsFor: 'analog pins' stamp: 'Richo 8/26/2009 16:09'!
analogRead: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'servos' stamp: 'Richo 8/26/2009 16:54'!
attachServoToPin: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'connection' stamp: 'Richo 1/31/2011 10:00'!
connectOnPort: portName boardType: anArduinoType
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'analog pins' stamp: 'Richo 8/26/2009 16:51'!
deactivateAnalogPin: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 9/11/2009 13:47'!
deactivateDigitalPort: portNumber
self subclassResponsibility
! !

!ArduinoProtocol methodsFor: 'servos' stamp: 'Richo 9/7/2009 16:25'!
detachServoFromPin: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 8/26/2009 16:53'!
digitalPin: pinNumber mode: mode
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 8/26/2009 16:51'!
digitalRead: pinNumber
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'digital pins' stamp: 'Richo 8/26/2009 17:06'!
digitalWrite: pinNumber value: value
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'connection' stamp: 'Richo 8/26/2009 16:08'!
disconnect
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'connection' stamp: 'Richo 8/26/2009 16:09'!
isConnected
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'servos' stamp: 'Richo 8/26/2009 16:54'!
servoOnPin: pinNumber angle: degrees
self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:52'!
valueForInputMode
"Return the internal (usually numeric) value used to represent the input mode"
^self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:53'!
valueForOutputMode
"Return the internal (usually numeric) value used to represent the output mode"
^self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'pin modes' stamp: 'Richo 4/30/2010 15:44'!
valueForPwmMode
"Return the internal (usually numeric) value used to represent the pwm mode"
^self subclassResponsibility! !

!ArduinoProtocol methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:53'!
valueForServoMode
"Return the internal (usually numeric) value used to represent the servo mode"
^self subclassResponsibility! !

ArduinoProtocol subclass: #Firmata
	instanceVariableNames: 'port analogInputData digitalInputData digitalOutputData parsingSysex storedInputData sysexBytesRead waitForData executeMultiByteCommand multiByteChannel majorVersion minorVersion steppingProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Firmata'!

!Firmata commentStamp: 'Richo 3/9/2010 11:03' prior: 0!
I am an implementation of the Firmata protocol for talking to an Arduino board. 
For more information check: http://www.firmata.org/

This implementation is mostly based on FirmataVB by Andrew Craigie.
http://www.acraigie.com/programming/firmatavb/default.html!

!Firmata class methodsFor: 'instance creation' stamp: 'Richo 1/31/2011 09:09'!
onPort: portNumber baudRate: aNumber
^self new connectOnPort: portNumber baudRate: aNumber! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 1/11/2010 18:36'!
activateAnalogPin: pinNumber 
	self analogPinReport: pinNumber onOff: 1.
! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 9/11/2009 13:46'!
activateDigitalPort: n
			self digitalPortReport: n onOff: 1
! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 11/11/2009 15:13'!
analogPinReport: pin onOff: onOff 
	"Sends a message to turn Analog Pin reportng on or off for a pin"
self port 
		nextPutAll: (ByteArray
				with: (FirmataConstants reportAnalog  bitOr: pin)
				with: onOff)! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 8/18/2009 12:35'!
analogRead: pinNumber
"Returns an analog value for an individual Analog pin"
^analogInputData at: pinNumber + 1! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 8/25/2009 14:47'!
analogWrite: pin value: value 
	"Sends a PWM value to a digital pin. 0 - 255"
	| analogWriteBuffer |
	analogWriteBuffer := ByteArray new: 3 withAll: 0.
	analogWriteBuffer
		at: 1
		put: (FirmataConstants analogMessage
				bitOr: (pin bitAnd: 15));
		
		at: 2
		put: (value rounded bitAnd: 127);
		
		at: 3
		put: ((value rounded bitShift: -7)
				bitAnd: 127).
self port  nextPutAll: analogWriteBuffer! !

!Firmata methodsFor: 'servos' stamp: 'Richo 9/1/2009 11:35'!
attachServoToPin: pinNumber 
	self
		servoConfig: pinNumber
		minPulse: 544
		maxPulse: 2400
		angle: 0! !

!Firmata methodsFor: 'connection' stamp: 'Richo 1/31/2011 10:01'!
connectOnPort: aString boardType: anArduinoType
	"If the port cant't be open it stores nil, this way we can know if it is
	connected by asking if port is nil.
	In windows we can't open a port using the port name, so I convert the
	string to an integer"
	port := SerialPort new baudRate: anArduinoType firmataRate.
	port := port
				openPort: (('COM*' match: aString)
						ifTrue: [aString asInteger]
						ifFalse: [aString]).
	self isConnected
		ifTrue: [self startSteppingProcess; controlFirmataInstallation]! !

!Firmata methodsFor: 'error control' stamp: 'Richo 4/6/2011 10:41'!
controlConnection
self isConnected ifFalse: [^self error: 'Serial port is not connected' translated].! !

!Firmata methodsFor: 'error control' stamp: 'Richo 1/31/2011 11:54'!
controlFirmataInstallation
	"If Firmata is not installed I throw an error"
	self isFirmataInstalled
		ifFalse: [^ FirmataNotInstalledError signal]! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 8/26/2009 17:03'!
deactivateAnalogPin: pinNumber 
	self analogPinReport: pinNumber onOff: 0! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 9/11/2009 13:46'!
deactivateDigitalPort: n
			self digitalPortReport: n onOff: 0
! !

!Firmata methodsFor: 'servos' stamp: 'Richo 9/7/2009 16:45'!
detachServoFromPin: pinNumber 
	self pin: pinNumber mode: FirmataConstants pinModeOutput ! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 1/12/2010 14:10'!
digitalPin: pinNumber mode: mode
"Digital ports should always be active"
pinNumber <= 7
	ifTrue: [self activateDigitalPort: 0]
	ifFalse: [self activateDigitalPort: 1].
self  pin: pinNumber mode: mode! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 8/31/2009 16:22'!
digitalPortReport: port1 onOff: onOff
"Sends a message to toggle reporting for a whole digital port"
	self port 
		nextPutAll: (ByteArray
				with: (FirmataConstants reportDigital bitOr: port1 )
				with: onOff)! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 9/14/2009 15:32'!
digitalRead: pinNumber 
	"Retuns On/Off data for an individual Digital Pin"
	^ ((digitalInputData at: (pinNumber bitShift: -3)
			+ 1)
		bitShift: (pinNumber bitAnd: 7) * -1)
		bitAnd: 1! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 8/18/2009 12:42'!
digitalWrite: pin value: value 
	"Sends an On or Off message to an individual Digital Pin"
	| portNumber adjustment digitalWriteBuffer |
	portNumber := (pin bitShift: -3)
				bitAnd: 15.
	adjustment := 1
				bitShift: (pin bitAnd: 7).
	digitalWriteBuffer := ByteArray new: 3 withAll: 0.
	value = 0
		ifTrue: [digitalOutputData
				at: portNumber + 1
				put: ((digitalOutputData at: portNumber + 1)
						bitAnd: adjustment bitInvert)]
		ifFalse: [digitalOutputData
				at: portNumber + 1
				put: ((digitalOutputData at: portNumber + 1)
						bitOr: adjustment)].
	digitalWriteBuffer
		at: 1
		put: (FirmataConstants digitalMessage bitOr: portNumber);
		at: 2
		put: ((digitalOutputData at: portNumber + 1)
				bitAnd: 127);
		at: 3
		put: ((digitalOutputData at: portNumber + 1)
				bitShift: -7).
	self port  nextPutAll: digitalWriteBuffer! !

!Firmata methodsFor: 'connection' stamp: 'Richo 3/9/2010 11:12'!
disconnect
	self isConnected
		ifTrue: [self stopSteppingProcess.
			port close.
			port := nil.
			self initializeVariables]! !

!Firmata methodsFor: 'other commands' stamp: 'Richo 8/18/2009 12:41'!
endSysex
"Sends an End Sysex message"
self port  nextPutAll: (ByteArray with: FirmataConstants endSysex  ).! !

!Firmata methodsFor: 'initialize-release' stamp: 'Richo 9/8/2009 12:40'!
initialize
self initializeVariables! !

!Firmata methodsFor: 'initialize-release' stamp: 'Richo 9/8/2009 12:40'!
initializeVariables
	digitalOutputData := Array new: 16 withAll: 0.
	digitalInputData := Array new: 16 withAll: 0.
	analogInputData := Array new: 16 withAll: 0.
	storedInputData := Array new: FirmataConstants maxDataBytes + 1 withAll: 0.
	parsingSysex := false.
	sysexBytesRead := waitForData := executeMultiByteCommand := multiByteChannel := majorVersion := minorVersion := 0! !

!Firmata methodsFor: 'connection' stamp: 'Richo 8/18/2009 12:40'!
isConnected
^port notNil! !

!Firmata methodsFor: 'testing' stamp: 'Richo 1/31/2011 11:53'!
isFirmataInstalled
"I check if Firmata responds well to queryVersion. I try for 5 seconds"
| t timeout |
t := Time millisecondClockValue.
timeout := 5000.
[self queryVersion. 500 milliSeconds asDelay wait.
self version > 0 or: [(Time millisecondsSince: t) > timeout]] whileFalse.
^self version > 0! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 1/5/2010 13:02'!
majorVersion
^majorVersion! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 8/18/2009 13:22'!
majorVersion: version1 minorVersion: version2
majorVersion := version1.
minorVersion := version2! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 1/5/2010 13:02'!
minorVersion
^minorVersion! !

!Firmata methodsFor: 'receiving' stamp: 'Richo 3/9/2010 11:34'!
parseCommandHeader: inputByte 
"I check the inputByte for a command header and then I set the variables accordingly to process the data in the following step"
	| command |
	inputByte < 240
		ifTrue: [command := inputByte bitAnd: 240.
			multiByteChannel := inputByte bitAnd: 15.]
		ifFalse: [command := inputByte].
	command caseOf: {
		[FirmataConstants analogMessage] -> 
			[waitForData := 2.
			executeMultiByteCommand := command].
		[FirmataConstants digitalMessage] -> 
			[waitForData := 2.
			executeMultiByteCommand := command].
		[FirmataConstants setPinMode] -> 
			[waitForData := 2.
			executeMultiByteCommand := command].
		[FirmataConstants reportDigital] -> 
			[waitForData := 1.
			executeMultiByteCommand := command].
		[FirmataConstants startSysex] -> 
			[parsingSysex := true.
			sysexBytesRead := 0].
		[FirmataConstants reportVersion] -> 
			[waitForData := 2.
			executeMultiByteCommand := command]}
		 otherwise: []! !

!Firmata methodsFor: 'receiving' stamp: 'Richo 3/9/2010 11:35'!
parseData: inputByte 
"I check inputByte for data according to the command header I found in the previous step, then I use the data to set the corresponding variables"

	waitForData := waitForData - 1.
	storedInputData at: waitForData + 1 put: inputByte.

	(waitForData = 0
			and: [executeMultiByteCommand ~= 0])
		ifTrue: [executeMultiByteCommand caseOf: {
				[FirmataConstants analogMessage] -> [self setAnalogInput: multiByteChannel value: (storedInputData first bitShift: 7)
						+ storedInputData second].
				[FirmataConstants digitalMessage] -> [
						self setDigitalInputs: multiByteChannel data: (storedInputData first bitShift: 7)
						+ storedInputData second].
				[FirmataConstants reportVersion] -> [self majorVersion: storedInputData first minorVersion: storedInputData second]}
				 otherwise: [].
			executeMultiByteCommand := 0]! !

!Firmata methodsFor: 'receiving' stamp: 'Richo 9/8/2009 10:31'!
parseSysex: inputByte 
	inputByte = FirmataConstants endSysex
		ifTrue: [parsingSysex := false]
		ifFalse: [storedInputData at: sysexBytesRead + 1 put: inputByte.
			sysexBytesRead := sysexBytesRead + 1
				]! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 1/12/2010 13:09'!
pin: pin mode: mode
"Sends an individual Pin Mode message"
self port nextPutAll: (ByteArray with: FirmataConstants setPinMode with: pin with: mode).! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 8/18/2009 12:41'!
port
self controlConnection.
^port! !

!Firmata methodsFor: 'printing' stamp: 'Richo 8/24/2009 15:14'!
printOn: aStream
super printOn: aStream.
aStream nextPutAll: '(',self hash asString,')'! !

!Firmata methodsFor: 'receiving' stamp: 'Richo 9/14/2009 16:26'!
processInput
	| data |
	data := port readByteArray.

	data
		do: [:inputByte | 

parsingSysex
				ifTrue: [self parseSysex: inputByte]
				ifFalse: [(waitForData > 0
							and: [inputByte < 128])
						ifTrue: [self parseData: inputByte]
						ifFalse: [self parseCommandHeader: inputByte]]]! !

!Firmata methodsFor: 'other commands' stamp: 'Richo 8/18/2009 12:42'!
queryFirmware
"Send a Firmware Name and Version query"
self startSysex.
self port  nextPutAll: (ByteArray with: FirmataConstants reportFirmware ).! !

!Firmata methodsFor: 'other commands' stamp: 'Richo 1/31/2011 10:06'!
queryVersion
"Send a version report query"
self port nextPutAll: (ByteArray with: FirmataConstants reportVersion ).! !

!Firmata methodsFor: 'servos' stamp: 'Richo 8/26/2009 16:14'!
servoConfig: pin minPulse: minPulse maxPulse: maxPulse angle: angle
	"Default values: 
	minPulse 544 (LSB 32 MSB 4)
	maxPulse 2400 (LSB 96 MSB 18
	"
	| b |
	b := ByteArray new: 10.
	b at: 1 put: FirmataConstants startSysex;
		 at: 2 put: FirmataConstants servoConfig;
		 at: 3 put: pin;
		 at: 4 put:  (minPulse rounded bitAnd: 127);
		 at: 5 put: ((minPulse rounded bitShift: -7)
				bitAnd: 127);
		 at: 6 put: (maxPulse rounded bitAnd: 127);
		 at: 7 put: ((maxPulse rounded bitShift: -7)
				bitAnd: 127);
		 at: 8 put: (angle rounded bitAnd: 127);
		 at: 9 put: ((angle rounded bitShift: -7)
				bitAnd: 127);
		 at: 10 put: FirmataConstants endSysex.
	self port nextPutAll: b! !

!Firmata methodsFor: 'servos' stamp: 'Richo 8/26/2009 17:00'!
servoOnPin: pinNumber angle: degrees
self analogWrite: pinNumber value: degrees! !

!Firmata methodsFor: 'analog pins' stamp: 'Richo 8/18/2009 13:01'!
setAnalogInput: pin value: value
"Stores analog value data in array"
	analogInputData at: pin + 1 put: value! !

!Firmata methodsFor: 'digital pins' stamp: 'Richo 8/18/2009 13:01'!
setDigitalInputs: portNumber data: portData 
"Puts but masked On/Off data for a whole digital port into array"
	digitalInputData at: portNumber + 1 put: portData! !

!Firmata methodsFor: 'stepping' stamp: 'Richo 3/9/2010 11:36'!
startSteppingProcess
	steppingProcess notNil
		ifTrue: [^ self].
	steppingProcess := [[self step.
			(Delay forMilliseconds: self stepTime) wait] repeat]
				forkNamed: self asString ! !

!Firmata methodsFor: 'other commands' stamp: 'Richo 8/18/2009 12:41'!
startSysex
"Sends a Start Sysex message"
self port  nextPutAll: (ByteArray with: FirmataConstants startSysex  ).! !

!Firmata methodsFor: 'stepping' stamp: 'Richo 9/9/2009 18:00'!
step
	[self processInput]
		on: Error
		do: [:err | 
			Transcript show: self asString , ': ' , err description;
				 cr. self initializeVariables.
			"self resetConnection"]! !

!Firmata methodsFor: 'stepping' stamp: 'Richo 8/31/2009 14:47'!
stepTime
	^ 1! !

!Firmata methodsFor: 'stepping' stamp: 'Richo 8/18/2009 15:23'!
stopSteppingProcess
	steppingProcess isNil
		ifTrue: [^ self].
	steppingProcess terminate. 
steppingProcess := nil.! !

!Firmata methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:47'!
valueForInputMode
^FirmataConstants pinModeInput! !

!Firmata methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:47'!
valueForOutputMode
^FirmataConstants pinModeOutput ! !

!Firmata methodsFor: 'pin modes' stamp: 'Richo 4/30/2010 15:44'!
valueForPwmMode
^FirmataConstants pinModePwm! !

!Firmata methodsFor: 'pin modes' stamp: 'Richo 3/9/2010 10:47'!
valueForServoMode
^FirmataConstants pinModeServo ! !

!Firmata methodsFor: 'accessing' stamp: 'Richo 3/30/2010 13:11'!
version
"self queryVersion."
^(majorVersion asString , '.', minorVersion asString) asNumber! !

Object subclass: #ArduinoType
	instanceVariableNames: 'fCpu friendlyNames mcu uploadRate firmataRate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core'!

ArduinoType class
	instanceVariableNames: 'types'!

ArduinoType class
	instanceVariableNames: 'types'!

!ArduinoType class methodsFor: 'accessing types' stamp: 'Richo 1/31/2011 09:55'!
allFriendlyNames
	^ types inject: #() into: [:subTotal :next | subTotal, next friendlyNames]! !

!ArduinoType class methodsFor: 'accessing types' stamp: 'Richo 1/31/2011 09:46'!
at: aSymbol
	^ types at: aSymbol! !

!ArduinoType class methodsFor: 'instance creation' stamp: 'Richo 1/31/2011 09:29'!
friendlyNames: anArray fCpu: anInteger mcu: aString uploadRate: uploadRate firmataRate: firmataRate
	^ (self basicNew setFriendlyNames: anArray fCpu: anInteger mcu: aString uploadRate: uploadRate firmataRate: firmataRate) initialize! !

!ArduinoType class methodsFor: 'class initialization' stamp: 'Richo 1/31/2011 09:42'!
initialize
"
self initialize
"
types := Dictionary new.

types at: #ArduinoDiecimila
	 put: (ArduinoType 
		friendlyNames: #('diecimila w/ atmega168' 'duemilanove w/ atmega168' 'nano w/ atmega168')
		fCpu: 16000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoLilyPad
	 put: (ArduinoType 
		friendlyNames: #('lilypad w/ atmega168')
		fCpu: 8000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoMega
	 put: (ArduinoType 
		friendlyNames: #('mega')
		fCpu: 16000000
		mcu: 'atmega1280'
		uploadRate: 57600
		firmataRate: 57600).

types at: #ArduinoBT
	 put: (ArduinoType 
		friendlyNames: #('bluetooth')
		fCpu: 16000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoATmega8
	 put: (ArduinoType 
		friendlyNames: #('ng or older w/ atmega8')
		fCpu: 16000000
		mcu: 'atmega8'
		uploadRate: 19200
		firmataRate: 115200).

types at: #ArduinoMini
	 put: (ArduinoType 
		friendlyNames: #('mini')
		fCpu: 16000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoATmega168
	 put: (ArduinoType 
		friendlyNames: #('ng or older w/ atmega168')
		fCpu: 16000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoLilyPad328
	 put: (ArduinoType 
		friendlyNames: #('lilypad w/ atmega328')
		fCpu: 8000000
		mcu: 'atmega328p'
		uploadRate: 57600
		firmataRate: 57600).

types at: #ArduinoATmega328
	 put: (ArduinoType 
		friendlyNames: #('duemilanove w/ atmega328' 'nano w/ atmega328')
		fCpu: 16000000
		mcu: 'atmega328p'
		uploadRate: 57600
		firmataRate: 57600).

types at: #ArduinoPro
	 put: (ArduinoType 
		friendlyNames: #('pro w/ atmega168' 'pro mini w/ atmega168')
		fCpu: 8000000
		mcu: 'atmega168'
		uploadRate: 19200
		firmataRate: 57600).

types at: #ArduinoPro328
	 put: (ArduinoType 
		friendlyNames: #('pro w/ atmega328' 'pro mini w/ atmega328')
		fCpu: 8000000
		mcu: 'atmega328p'
		uploadRate: 57600
		firmataRate: 57600).! !

!ArduinoType class methodsFor: 'accessing types' stamp: 'Richo 4/6/2011 10:43'!
withFriendlyName: aSymbol 
"Return an ArduinoType with the friendly name given as a parameter"

	^ types detect: [:each | each friendlyNames includes: aSymbol] ifNone: [self error: 'ArduinoType not found' translated]! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:45'!
avrdudeProgrammer
	^'stk500'! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
fCpu
	"Answer the value of fCpu"

	^ fCpu! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
firmataRate
	"Answer the value of firmataRate"

	^ firmataRate! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
friendlyNames
	"Answer the value of friendlyNames"

	^ friendlyNames! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
mcu
	"Answer the value of mcu"

	^ mcu! !

!ArduinoType methodsFor: 'private' stamp: 'Richo 1/31/2011 09:31'!
setFriendlyNames: anArray fCpu: anInteger mcu: aString uploadRate: uploadRate1 firmataRate: firmataRate1
	friendlyNames := anArray.
	fCpu := anInteger.
	mcu := aString.
	uploadRate := uploadRate1.
	firmataRate := firmataRate1! !

!ArduinoType methodsFor: 'accessing' stamp: 'Richo 1/31/2011 09:31'!
uploadRate
	"Answer the value of uploadRate"

	^ uploadRate! !

Object subclass: #AttachableDevice
	instanceVariableNames: 'pins isConfigured'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!AttachableDevice commentStamp: '<historical>' prior: 0!
Abstract class. I represent a device that can be attached to a collection of Arduino's pins. My protocol is very simple:

#attach: aPin
Attaches the device to a single pin and sets the pins configuration accordingly.

#isAttached
Checks if the device is attached to a pin.

#detach
Simply detachs the device from the pins and deletes the configuration. It raises an error if the device is not attached.

Subclasses should override #configure and #deleteConfiguration with the specific steps to configure the corresponding device.!

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 14:20'!
attach: aPin
	self setPins: (Array with: aPin)! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 15:08'!
attachAnalogPins: anArray on: anArduino 
"
	self attachAnalogPins: #(1 2 3 4) on: Arduino new.
"
	self setPins: (anArray collect: [:pinNumber| anArduino analogPin: pinNumber]).! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 15:08'!
attachDigitalPins: anArray on: anArduino 
"
	self attachDigitalPins: #(1 2 3 4) on: Arduino new.
"
	self setPins: (anArray collect: [:pinNumber| anArduino digitalPin: pinNumber]).! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 3/9/2010 10:59'!
configure
"This method is responsible of configuring the pin for using the corresponding device. Most of the times the configuration is simply setting the correct pin mode"
self subclassResponsibility! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 9/8/2009 12:52'!
controlAttaching
self isAttached
		ifFalse: [^ self unattachedError].! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 3/9/2010 11:00'!
deleteConfiguration
"This method simply resets the pin's configuration to its default state"
	self subclassResponsibility! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 14:21'!
detach
	self controlAttaching.
	[self deleteConfiguration] on: Error do: ["The port may be disconnected"].
	pins := nil! !

!AttachableDevice methodsFor: 'attaching' stamp: 'Richo 6/1/2010 14:21'!
isAttached
^pins notNil! !

!AttachableDevice methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:21'!
reconfigureIfNecessary
	"This method must be called every time we try to communicate with the
	device. This way we are sure that the device is properly configured"
	isConfigured
		ifFalse: [self setPins: pins]! !

!AttachableDevice methodsFor: 'private' stamp: 'Richo 6/1/2010 14:21'!
setPins: anArrayOfPins
"When attaching a device I try to configure it, but if it fails for some reason I simply set isConfigured to false, so that we remember to configure whenever possible"
	pins := anArrayOfPins.
	[self configure.
	isConfigured := true]
		on: Error
		do: [isConfigured := false]! !

!AttachableDevice methodsFor: 'errors' stamp: 'Richo 4/6/2011 10:40'!
unattachedError
^self error: 'Unattached device: ' translated, self asString.! !

AttachableDevice subclass: #Buzzer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Buzzer methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:22'!
configure
	pins first mode: PwmMode new! !

!Buzzer methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:22'!
deleteConfiguration
	pins first setOutput! !

!Buzzer methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:22'!
value
	self controlAttaching;  reconfigureIfNecessary.
	^ pins first value! !

!Buzzer methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:23'!
value: aNumber 
	"aNumber can be 0 - 255"
	self controlAttaching;reconfigureIfNecessary.
	pins first value: (aNumber min: 255 max: 0)! !

AttachableDevice subclass: #DCMotor
	instanceVariableNames: 'speed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!DCMotor methodsFor: 'accessing' stamp: 'Richo 6/27/2011 15:34'!
brake
	self d0Pin value: 1.
	self d1Pin value: 1.
	self enablePin value: 255! !

!DCMotor methodsFor: 'configuring' stamp: 'Richo 6/27/2011 15:28'!
configure 
	pins with: {OutputMode. OutputMode. PwmMode} do: [:pin :mode |
		pin mode: mode new;
			value: 0].! !

!DCMotor methodsFor: 'private' stamp: 'Richo 6/27/2011 15:26'!
d0Pin
	^ pins first! !

!DCMotor methodsFor: 'private' stamp: 'Richo 6/27/2011 15:26'!
d1Pin
	^ pins second! !

!DCMotor methodsFor: 'configuring' stamp: 'Richo 6/27/2011 15:16'!
deleteConfiguration
	pins do: [:each | each setOutput]! !

!DCMotor methodsFor: 'private' stamp: 'Richo 6/27/2011 15:26'!
enablePin
	^ pins third! !

!DCMotor methodsFor: 'initialize-release' stamp: 'Richo 6/27/2011 15:33'!
initialize
	super initialize.
	speed := 0.! !

!DCMotor methodsFor: 'accessing' stamp: 'Richo 6/27/2011 15:33'!
speed
	^ speed! !

!DCMotor methodsFor: 'accessing' stamp: 'Richo 6/27/2011 15:32'!
speed: aNumber
	speed := aNumber min: 100 max: -100.
	speed < 0
		ifTrue: [self d0Pin value: 0.
			self d1Pin value: 1]
		ifFalse: [self d0Pin value: 1.
			self d1Pin value: 0].
	self enablePin value: (speed abs / 100 * 255)
	! !

AttachableDevice subclass: #EButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!EButton commentStamp: 'Richo 9/15/2009 12:07' prior: 0!
I represent a button (digital input device).!

!EButton methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:23'!
configure
	pins first mode: ButtonMode new! !

!EButton methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:23'!
deleteConfiguration
pins first setOutput! !

!EButton methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:23'!
value
self controlAttaching ; reconfigureIfNecessary .
^pins first value! !

AttachableDevice subclass: #Keypad
	instanceVariableNames: 'buttons steppingProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Keypad methodsFor: 'accessing' stamp: 'Richo 6/1/2010 15:13'!
at: rowNumber at: columnNumber
	^self atRow: rowNumber column: columnNumber! !

!Keypad methodsFor: 'accessing' stamp: 'Richo 6/1/2010 15:13'!
atRow: rowNumber column: columnNumber
	self controlAttaching; reconfigureIfNecessary.
	^buttons at: rowNumber at: columnNumber! !

!Keypad methodsFor: 'private' stamp: 'Richo 5/31/2010 18:18'!
columnPins
^pins first: 3! !

!Keypad methodsFor: 'configuring' stamp: 'Richo 5/31/2010 17:58'!
configure
self rowPins do: [:each | each setOutput].
self columnPins do: [:each | each setInput].
self startSteppingProcess.! !

!Keypad methodsFor: 'configuring' stamp: 'Richo 5/31/2010 18:13'!
deleteConfiguration
self columnPins , self rowPins do: [:each | each setOutput].
self stopSteppingProcess ! !

!Keypad methodsFor: 'initialize-release' stamp: 'Richo 6/8/2010 15:35'!
finalize
	self stopSteppingProcess.! !

!Keypad methodsFor: 'initialize-release' stamp: 'Richo 6/8/2010 17:28'!
initialize
	super initialize.
	self finalizationRegistry add: self.
	buttons := Matrix rows: 3 columns: 3 element: false.! !

!Keypad methodsFor: 'private' stamp: 'Richo 5/31/2010 18:00'!
rowPins
^pins last: 3! !

!Keypad methodsFor: 'attaching' stamp: 'Richo 5/31/2010 18:04'!
rowPins: rowPinNumbers columnsPins: columnPinNumbers on: anArduino
"
	self rowPins: #(7 6 5) columnPins: #(2 3 4) on: Arduino new.
"
self attachDigitalPins: columnPinNumbers , rowPinNumbers on: anArduino! !

!Keypad methodsFor: 'stepping' stamp: 'Richo 6/8/2010 17:25'!
startSteppingProcess
	steppingProcess notNil
		ifTrue: [^ self].
	steppingProcess := [[self step] repeat] forkNamed: self asString ! !

!Keypad methodsFor: 'stepping' stamp: 'Richo 5/31/2010 18:27'!
step
1 to: 3 do: [:r |
	[self rowPins first value: (r = 1) asBit.
	self rowPins second value: (r = 2) asBit.
	self rowPins third value: (r = 3) asBit.
	50 milliSeconds asDelay wait.
	1 to: 3 do: [:c |
		buttons at: r at: c put: ((self columnPins at: c) value = 1).
	]] on: Error do: [200 milliSeconds asDelay wait]]! !

!Keypad methodsFor: 'stepping' stamp: 'Richo 5/31/2010 17:59'!
stepTime
	^ 1! !

!Keypad methodsFor: 'stepping' stamp: 'Richo 5/31/2010 17:59'!
stopSteppingProcess
	steppingProcess isNil
		ifTrue: [^ self].
	steppingProcess terminate. 
	steppingProcess := nil.! !

AttachableDevice subclass: #LightEmittingDiode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!LightEmittingDiode commentStamp: 'Richo 9/15/2009 12:08' prior: 0!
I represent a led (digital output device).!

!LightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:23'!
configure
	pins first mode: LedMode new! !

!LightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:23'!
deleteConfiguration
	pins first setOutput! !

!LightEmittingDiode methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:23'!
value
	self controlAttaching;  reconfigureIfNecessary.
	^ pins first value! !

!LightEmittingDiode methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:23'!
value: aNumber 
	self controlAttaching;reconfigureIfNecessary.
	pins first value: aNumber! !

AttachableDevice subclass: #Photoresistor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Photoresistor commentStamp: 'Richo 9/15/2009 12:08' prior: 0!
I represent an LDR (analog input device)!

!Photoresistor methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:23'!
configure

	pins first activate! !

!Photoresistor methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:23'!
deleteConfiguration

	pins first deactivate! !

!Photoresistor methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:24'!
value
	self controlAttaching; reconfigureIfNecessary.
	^pins first value! !

AttachableDevice subclass: #Potentiometer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Potentiometer commentStamp: 'Richo 9/15/2009 12:09' prior: 0!
I represent a Potentiometer (analog input device).!

!Potentiometer methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:24'!
configure
pins first activate! !

!Potentiometer methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:24'!
deleteConfiguration

	pins first deactivate! !

!Potentiometer methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:24'!
value
	self controlAttaching; reconfigureIfNecessary.
	^ pins first value! !

AttachableDevice subclass: #PwmLightEmittingDiode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!PwmLightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:24'!
configure
	pins first mode: PwmMode new! !

!PwmLightEmittingDiode methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:24'!
deleteConfiguration
	pins first setOutput! !

!PwmLightEmittingDiode methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:24'!
value
	self controlAttaching;  reconfigureIfNecessary.
	^ pins first value! !

!PwmLightEmittingDiode methodsFor: 'accessing' stamp: 'Richo 6/1/2010 15:06'!
value: aNumber 
	"aNumber can be 0 - 255"
	self controlAttaching;reconfigureIfNecessary.
	pins first value: (aNumber min: 255 max: 0)! !

AttachableDevice subclass: #Servo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Servo commentStamp: 'Richo 9/15/2009 12:11' prior: 0!
I represent a servo motor.!

!Servo methodsFor: 'accessing' stamp: 'Richo 6/1/2010 15:21'!
angle
	self controlAttaching; reconfigureIfNecessary.
	^pins first value! !

!Servo methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:24'!
angle: aSmallInteger 
	self controlAttaching; reconfigureIfNecessary.
	pins first value: (aSmallInteger min: 180 max: 0)! !

!Servo methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:24'!
configure
	pins first mode: ServoMode new.! !

!Servo methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:24'!
deleteConfiguration
	pins first setOutput.! !

!Servo methodsFor: 'initialize-release' stamp: 'Richo 4/30/2010 16:57'!
initialize
super initialize.! !

AttachableDevice subclass: #StepperMotor
	instanceVariableNames: 'steppingProcess stepTime stepMode isRunning isGoingForward currentStep'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!StepperMotor methodsFor: 'configuring' stamp: 'Richo 6/8/2010 15:20'!
configure
	pins do: [:each | each setOutput; value: 0].
! !

!StepperMotor methodsFor: 'configuring' stamp: 'Richo 6/8/2010 15:20'!
deleteConfiguration
! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 6/9/2010 16:41'!
doNextStep
	[	stepMode value.
	currentStep := currentStep + (isGoingForward ifTrue: [1] ifFalse: [-1])
] on: Error do: [200 milliSeconds asDelay wait]! !

!StepperMotor methodsFor: 'initialize-release' stamp: 'Richo 6/8/2010 17:29'!
finalize
	self stopSteppingProcess.
! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 17:05'!
goBackwards
	isGoingForward := false! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 17:05'!
goForward
	isGoingForward := true! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 17:00'!
halfStepMode
^[ | values |
	values := (self halfStepModeSteps at: currentStep \\ 8 + 1).
	1 to: 4 do: [:i | (pins at: i) value: (values at: i)].
]! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 16:33'!
halfStepModeSteps
^#(
	#(1 0 0 0)
	#(1 1 0 0)
	#(0 1 0 0)
	#(0 1 1 0)
	#(0 0 1 0)
	#(0 0 1 1)
	#(0 0 0 1)
	#(1 0 0 1)
)! !

!StepperMotor methodsFor: 'initialize-release' stamp: 'Richo 6/8/2010 16:43'!
initialize
super initialize.
self finalizationRegistry add: self.
stepTime := 100.
isGoingForward := true.
isRunning := false.
currentStep := 1.
self setNormalMode.
self startSteppingProcess ! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 6/9/2010 15:54'!
isGoingForward
^isGoingForward ! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 6/9/2010 16:37'!
isRunning
^isRunning! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 6/9/2010 15:54'!
isUsingHalfStepMode
^stepMode = self waveDriveMode! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 6/9/2010 15:54'!
isUsingNormalMode
^stepMode = self normalMode! !

!StepperMotor methodsFor: 'accessing' stamp: 'Richo 6/9/2010 15:54'!
isUsingWaveDriveMode
^stepMode = self waveDriveMode! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 17:00'!
normalMode
^[ | values |
	values := (self normalModeSteps at: currentStep \\ 4 + 1).
	1 to: 4 do: [:i | (pins at: i) value: (values at: i)].
]! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 16:32'!
normalModeSteps
^#(
	#(1 1 0 0)
	#(0 1 1 0)
	#(0 0 1 1)
	#(1 0 0 1)
)! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 17:05'!
reverse
	isGoingForward := isGoingForward not! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 16:40'!
setHalfStepMode
stepMode := self halfStepMode! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 16:27'!
setNormalMode
stepMode := self normalMode! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/8/2010 16:38'!
setWaveDriveMode
stepMode := self waveDriveMode! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/7/2010 18:21'!
start
	self controlAttaching; reconfigureIfNecessary.
	isRunning := true.! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 6/7/2010 18:20'!
startSteppingProcess
	steppingProcess notNil
		ifTrue: [^ self].
	steppingProcess := [[self step.
			(Delay forMilliseconds: self stepTime) wait] repeat]
				forkNamed: self asString ! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 6/9/2010 16:41'!
step
	isRunning ifFalse: [^self].
	self doNextStep! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 6/7/2010 18:07'!
stepTime
	^stepTime! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 4/6/2011 11:06'!
stepTime: aNumber
	aNumber < 1 ifTrue: [^self error: 'Step time can''t be less than 1' translated].
	stepTime := aNumber.! !

!StepperMotor methodsFor: 'actions' stamp: 'Richo 6/7/2010 18:22'!
stop
	self controlAttaching; reconfigureIfNecessary.
	isRunning := false.! !

!StepperMotor methodsFor: 'stepping' stamp: 'Richo 6/7/2010 18:21'!
stopSteppingProcess
	steppingProcess isNil
		ifTrue: [^ self].
	steppingProcess terminate. 
	steppingProcess := nil.! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 17:00'!
waveDriveMode
^[ | values |
	values := (self waveDriveModeSteps at: currentStep \\ 4 + 1).
	1 to: 4 do: [:i | (pins at: i) value: (values at: i)].
]! !

!StepperMotor methodsFor: 'step modes' stamp: 'Richo 6/8/2010 16:32'!
waveDriveModeSteps
^#(
	#(1 0 0 0)
	#(0 1 0 0)
	#(0 0 1 0)
	#(0 0 0 1)
)! !

AttachableDevice subclass: #Thermistor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!Thermistor methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:25'!
configure
pins first activate! !

!Thermistor methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:25'!
deleteConfiguration

	pins first deactivate! !

!Thermistor methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:25'!
value
	self controlAttaching; reconfigureIfNecessary.
	^pins first value! !

AttachableDevice subclass: #TiltSwitch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Devices'!

!TiltSwitch methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:25'!
configure
	pins first mode: TiltMode new! !

!TiltSwitch methodsFor: 'configuring' stamp: 'Richo 6/1/2010 14:25'!
deleteConfiguration
pins first setOutput! !

!TiltSwitch methodsFor: 'accessing' stamp: 'Richo 6/1/2010 14:25'!
value
self controlAttaching ; reconfigureIfNecessary .
^pins first value! !

Object subclass: #FirmataConstants
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-Firmata'!

!FirmataConstants commentStamp: 'Richo 9/15/2009 12:00' prior: 0!
See class side. I collect a bunch of constants for the Firmata protocol.!

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:35'!
analogMessage
"0xE0 send data for an analog pin (or PWM)"
^16rE0! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:35'!
digitalMessage
"0x90 send data for digital pin"
^16r90! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:39'!
endSysex
"0xF7 end a MIDI Sysex message"
^16rF7! !

!FirmataConstants class methodsFor: 'version numbers' stamp: 'Richo 8/18/2009 10:33'!
firmataMajorVersion
"for non-compatible changes"
^2! !

!FirmataConstants class methodsFor: 'version numbers' stamp: 'Richo 8/18/2009 10:33'!
firmataMinorVersion
"for backward compatible changes"
^0! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:41'!
firmataString
"0x71 a string message with 14-bits per character"
^16r71! !

!FirmataConstants class methodsFor: 'others' stamp: 'Richo 8/18/2009 10:34'!
maxDataBytes
"Maximum number of data bytes in non-Sysex messages"
^32 ! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:16'!
pinModeAnalog
"analog pin in analogInput mode"
^2! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:16'!
pinModeInput
^0! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:16'!
pinModeOutput
^1! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:17'!
pinModePwm
"digital pin in PWM output mode"
^3! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/7/2009 16:17'!
pinModeServo 
"digital pin in Servo output mode"
^4! !

!FirmataConstants class methodsFor: 'pin modes' stamp: 'Richo 9/11/2009 14:37'!
pinModeUnavailable
	^ -1! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:36'!
reportAnalog
"0xC0 enable analog input by pin number"
^16rC0! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:36'!
reportDigital
"0xD0 enable digital input by port pair"
^16rD0! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:41'!
reportFirmware
"0x79 report name and version of the firmware"
^16r79! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:38'!
reportVersion
"0xF9 report protocol version"
^16rF9! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:40'!
servoConfig
"0x70 set maximum angle, minPulse, maxPulse, frequency"
^16r70! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:37'!
setPinMode
"0xF4 set a pin to INPUT/OUTPUT/ANALOG/PWM/SERVO - 0/1/2/3/4"
^16rF4! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:38'!
startSysex
"0xF0 start a MIDI Sysex message"
^16rF0! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:41'!
sysexNonRealtime
"0x7E MIDI reserved for non-realtime messages"
^16r7E! !

!FirmataConstants class methodsFor: 'extended command set using Sysex' stamp: 'Richo 8/18/2009 10:42'!
sysexRealtime
"0x7F MIDI reserved for realtime messages"
^16r7F! !

!FirmataConstants class methodsFor: 'message commands' stamp: 'Richo 8/18/2009 10:38'!
systemReset
"0xFF reset from MIDI"
^16rFF! !

!FirmataConstants class methodsFor: 'version numbers' stamp: 'Richo 8/18/2009 10:33'!
versionBlickPin
"digital pin to blick version on"
^13! !

Object subclass: #PinMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

PinMode class
	instanceVariableNames: 'uniqueInstance'!

!PinMode commentStamp: 'Richo 9/15/2009 12:15' prior: 0!
Abstract class. I represent the mode of an Arduino digital pin. Tipically, I would just need input/output mode, other subclasses were made for practical purposes (i.e. ButtonMode, LedMode).!

PinMode class
	instanceVariableNames: 'uniqueInstance'!

PinMode subclass: #InputMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!InputMode commentStamp: 'Richo 9/15/2009 12:13' prior: 0!
I represent the input mode of an Arduino digital pin.!

InputMode subclass: #ButtonMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!ButtonMode commentStamp: 'Richo 9/15/2009 12:13' prior: 0!
I represent the button mode of an Arduino digital pin.!

!ButtonMode methodsFor: 'accessing' stamp: 'Richo 9/8/2009 12:58'!
nameSymbol
^#button! !

!InputMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:35'!
nameSymbol
	^#input! !

!InputMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:08'!
refreshModeOn: pinNumber using: anArduinoProtocol
	anArduinoProtocol digitalPin: pinNumber mode: anArduinoProtocol valueForInputMode.
! !

!InputMode methodsFor: 'refreshing' stamp: 'Richo 4/6/2011 10:42'!
refreshValue: value on: pinNumber using: anArduinoProtocol
^self error: 'Input mode cannot set value' translated! !

InputMode subclass: #TiltMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!TiltMode methodsFor: 'as yet unclassified' stamp: 'Richo 4/30/2010 18:10'!
nameSymbol
	^#tilt! !

PinMode subclass: #OutputMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!OutputMode commentStamp: 'Richo 9/15/2009 12:13' prior: 0!
I represent the output mode of an Arduino digital pin.!

OutputMode subclass: #LedMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!LedMode commentStamp: 'Richo 9/15/2009 12:14' prior: 0!
I represent the led mode of an Arduino digital pin.!

!LedMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:35'!
nameSymbol
	^#led! !

!OutputMode methodsFor: 'testing' stamp: 'Richo 9/7/2009 16:29'!
isOutput
^true! !

!OutputMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:34'!
nameSymbol
	^#output! !

!OutputMode methodsFor: 'refresing' stamp: 'Richo 4/30/2010 16:08'!
refreshModeOn: pinNumber using: anArduinoProtocol
	anArduinoProtocol digitalPin: pinNumber mode: anArduinoProtocol valueForOutputMode.
! !

!OutputMode methodsFor: 'refresing' stamp: 'Richo 4/30/2010 16:01'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	anArduinoProtocol digitalWrite: pinNumber value: value! !

OutputMode subclass: #PwmMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!PwmMode commentStamp: '<historical>' prior: 0!
I represent the pwm mode of an Arduino digital pin.!

!PwmMode methodsFor: 'accessing' stamp: 'Richo 4/30/2010 15:43'!
nameSymbol
	^#pwm! !

!PwmMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:08'!
refreshModeOn: pinNumber using: anArduinoProtocol
	anArduinoProtocol digitalPin: pinNumber mode: anArduinoProtocol valueForPwmMode.
! !

!PwmMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:03'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	anArduinoProtocol analogWrite: pinNumber value: value! !

OutputMode subclass: #ServoMode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-Core-PinMode'!

!ServoMode commentStamp: 'Richo 9/15/2009 12:13' prior: 0!
I represent the servo mode of an Arduino digital pin.!

!ServoMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:35'!
nameSymbol
	^#servo! !

!ServoMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:13'!
refreshModeOn: pinNumber using: anArduinoProtocol
	anArduinoProtocol attachServoToPin: pinNumber;
						digitalPin: pinNumber mode: anArduinoProtocol valueForServoMode.
! !

!ServoMode methodsFor: 'refreshing' stamp: 'Richo 4/30/2010 16:43'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	anArduinoProtocol servoOnPin: pinNumber angle: value! !

!PinMode class methodsFor: 'as yet unclassified' stamp: 'Richo 9/7/2009 16:23'!
new
^uniqueInstance ifNil: [uniqueInstance := self basicNew]! !

!PinMode methodsFor: 'testing' stamp: 'Richo 9/7/2009 16:29'!
isOutput
^false! !

!PinMode methodsFor: 'accessing' stamp: 'Richo 9/7/2009 16:33'!
nameSymbol
^#undefined! !

!PinMode methodsFor: 'setting pins' stamp: 'Richo 4/30/2010 16:07'!
refreshModeOn: pinNumber using: anArduinoProtocol
	self subclassResponsibility! !

!PinMode methodsFor: 'setting pins' stamp: 'Richo 4/30/2010 16:00'!
refreshValue: value on: pinNumber using: anArduinoProtocol
	self subclassResponsibility! !
ArduinoType initialize!
